#include "dto.h"

namespace dto::Responses {
    drogon::HttpResponsePtr get_error_res(drogon::HttpStatusCode code, const std::string& msg) {
        Json::Value json;
        json["statusCode"] = static_cast<int>(code);
        json["message"] = msg;
        auto res = drogon::HttpResponse::newHttpJsonResponse(json);
        res->setStatusCode(code);
        return res;
    }

    drogon::HttpResponsePtr get_success_res() {
        Json::Value json;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_success_res(Json::Value& json) {
        json["statusCode"] = 200;
        auto res = drogon::HttpResponse::newHttpJsonResponse(json);
        res->setStatusCode(drogon::HttpStatusCode::k200OK);
        return res;
    }

    drogon::HttpResponsePtr get_login_res(const std::string &jwt) {
        Json::Value json;
        json["jwt"] = jwt;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_tfa_setup_res(const std::string& secret, const std::string& qrcode) {
        Json::Value json;
        json["secret"] = secret;
        json["qrCode"] = qrcode;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_user_info_res(const std::string &name, bool gitlab, bool tfa) {
        Json::Value json;
        json["name"] = name;
        json["gitlab"] = gitlab;
        json["tfaEnabled"] = tfa;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_admin_users_res(const std::vector<GetUsersEntry>& users) {
        Json::Value json;
        for (const GetUsersEntry& user : users) {
            Json::Value entry;
            entry["id"] = user.id;
            entry["gitlab"] = user.gitlab;
            entry["name"] = user.name;
            entry["role"] = user.role;
            entry["tfaEnabled"] = user.tfa;
            json["users"].append(entry);
        }
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_root_res(uint64_t root) {
        Json::Value json;
        json["rootId"] = root;
        return get_success_res(json);
    }

    Json::Value parse_node(const GetNodeEntry& node) {
        Json::Value json;
        json["id"] = node.id;
        json["name"] = node.name;
        json["isFile"] = node.is_file;
        json["preview"] = node.has_preview;
        json["parent"] = (node.parent != nullptr) ? *node.parent : Json::Value::nullSingleton();
        if (node.is_file) json["size"] = node.size;
        return json;
    }

    drogon::HttpResponsePtr get_node_res(const GetNodeEntry& node, const std::vector<GetNodeEntry>& children) {
        Json::Value json = parse_node(node);
        if (!node.is_file) {
            json["children"] = Json::Value(Json::arrayValue);
            for (const GetNodeEntry& child : children)
                json["children"].append(parse_node(child));
        }
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_new_node_res(uint64_t id) {
        Json::Value json;
        json["id"] = id;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_node_exists_res(uint64_t id, bool file) {
        Json::Value json;
        json["id"] = id;
        json["exists"] = true;
        json["isFile"] = file;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_download_base64_res(const std::string &data) {
        Json::Value json;
        json["data"] = data;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_type_res(const std::string &type) {
        Json::Value json;
        json["type"] = type;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_create_zip_done_res() {
        Json::Value json;
        json["done"] = true;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_create_zip_done_res(uint64_t progress, uint64_t total) {
        Json::Value json;
        json["done"] = false;
        json["progress"] = progress;
        json["total"] = total;
        return get_success_res(json);
    }
}
